from matrix.constants import UNIVERSAL_PERMUTATION_METHODS, DECOMPOSITION_TYPES, NO_PERMUTATION_METHOD

CHOLMOD_NO_PERMUTATION_METHOD = 'natural'
try:
    import sksparse.cholmod
except ImportError:
    CHOLMOD_PERMUTATION_METHODS = ()
else:
    CHOLMOD_PERMUTATION_METHODS = tuple(sksparse.cholmod._ordering_methods.keys())
    assert 'best' in CHOLMOD_PERMUTATION_METHODS
    assert 'default' in CHOLMOD_PERMUTATION_METHODS
    assert 'amd' in CHOLMOD_PERMUTATION_METHODS
    assert 'metis' in CHOLMOD_PERMUTATION_METHODS
    assert 'nesdis' in CHOLMOD_PERMUTATION_METHODS
    assert 'colamd' in CHOLMOD_PERMUTATION_METHODS
    assert CHOLMOD_NO_PERMUTATION_METHOD in CHOLMOD_PERMUTATION_METHODS

FILL_REDUCE_PERMUTATION_METHOD_PREFIX = 'fill_reduce_'
FILL_REDUCE_PERMUTATION_METHODS = tuple(FILL_REDUCE_PERMUTATION_METHOD_PREFIX + method for method in CHOLMOD_PERMUTATION_METHODS if method != CHOLMOD_NO_PERMUTATION_METHOD)
SPARSE_ONLY_PERMUTATION_METHODS = FILL_REDUCE_PERMUTATION_METHODS
""" Supported permutation methods only for sparse matrices. """

if len(CHOLMOD_PERMUTATION_METHODS) > 0:
    BEST_FILL_REDUCE_PERMUTATION_METHOD = FILL_REDUCE_PERMUTATION_METHOD_PREFIX + 'best'
    """ Supported permutation method to best reduce fill of decomposed sparse matrices. """
    assert BEST_FILL_REDUCE_PERMUTATION_METHOD in FILL_REDUCE_PERMUTATION_METHODS
    DEFAULT_FILL_REDUCE_PERMUTATION_METHOD = FILL_REDUCE_PERMUTATION_METHOD_PREFIX + 'default'
    """ Supported permutation method to default reduce fill of decomposed sparse matrices. """
    assert DEFAULT_FILL_REDUCE_PERMUTATION_METHOD in FILL_REDUCE_PERMUTATION_METHODS
    AMD_FILL_REDUCE_PERMUTATION_METHOD = FILL_REDUCE_PERMUTATION_METHOD_PREFIX + 'amd'
    assert AMD_FILL_REDUCE_PERMUTATION_METHOD in FILL_REDUCE_PERMUTATION_METHODS
    METIS_FILL_REDUCE_PERMUTATION_METHOD = FILL_REDUCE_PERMUTATION_METHOD_PREFIX + 'metis'
    assert METIS_FILL_REDUCE_PERMUTATION_METHOD in FILL_REDUCE_PERMUTATION_METHODS
    NESDIS_FILL_REDUCE_PERMUTATION_METHOD = FILL_REDUCE_PERMUTATION_METHOD_PREFIX + 'nesdis'
    assert NESDIS_FILL_REDUCE_PERMUTATION_METHOD in FILL_REDUCE_PERMUTATION_METHODS
    COLAMD_FILL_REDUCE_PERMUTATION_METHOD = FILL_REDUCE_PERMUTATION_METHOD_PREFIX + 'colamd'
    assert COLAMD_FILL_REDUCE_PERMUTATION_METHOD in FILL_REDUCE_PERMUTATION_METHODS
else:
    BEST_FILL_REDUCE_PERMUTATION_METHOD = NO_PERMUTATION_METHOD
    DEFAULT_FILL_REDUCE_PERMUTATION_METHOD = NO_PERMUTATION_METHOD
    AMD_FILL_REDUCE_PERMUTATION_METHOD = NO_PERMUTATION_METHOD
    METIS_FILL_REDUCE_PERMUTATION_METHOD = NO_PERMUTATION_METHOD
    NESDIS_FILL_REDUCE_PERMUTATION_METHOD = NO_PERMUTATION_METHOD
    COLAMD_FILL_REDUCE_PERMUTATION_METHOD = NO_PERMUTATION_METHOD

PERMUTATION_METHODS = UNIVERSAL_PERMUTATION_METHODS + SPARSE_ONLY_PERMUTATION_METHODS
""" Supported permutation methods for decompose sparse matrices. """
